import pytest
import subprocess

class MockSubprocess(object):
    def __init__(self):
        """Constructor."""

        self.commands = {}

    def mock_command(self, cmd_string, exit_code=0, stdout=None, stderr=None):
        """Mock the command given by cmd_string to return a fixed response.

        Args:
            cmd_string (string): The command that should be mocked including arguments,
                must match exactly.
            exit_code (int): The exit code that should be returned
            stdout (string): The string that should be returned on stdout
            stderr (string): The string that should be returned on stderr
        """

        if stdout is None:
            stdout = ""

        if stderr is None:
            stderr = ""

        self.commands[cmd_string] = {
            'exit_code': exit_code,
            'stdout': stdout,
            'stderr': stderr
        }

    def check_output(self, args, stdin=None, stdout=None, stderr=None, shell=False):
        """Mock check_output call."""

        cmd_str = " ".join(args)
        if cmd_str not in self.commands:
            raise RuntimeError("Could not find mock command by name: %s" % cmd_str)

        cmd = self.commands[cmd_str]

        if stdout is not None:
            stdout.write(cmd['stdout'])

        if stderr is not None:
            stderr.write(cmd['stderr'])

        if cmd['exit_code'] != 0:
            raise subprocess.CalledProcessError(cmd['exit_code'], cmd_str, output=cmd['stdout'])

        return cmd['stdout']

    def check_call(self, args, stdin=None, stdout=None, stderr=None, shell=False):
        """Mock check_call call."""

        cmd_str = " ".join(args)
        if cmd_str not in self.commands:
            raise RuntimeError("Could not find mock command by name: %s" % cmd_str)

        cmd = self.commands[cmd_str]

        if stdout is not None:
            stdout.write(cmd['stdout'])

        if stderr is not None:
            stderr.write(cmd['stderr'])

        if cmd['exit_code'] != 0:
            raise subprocess.CalledProcessError(cmd['exit_code'], cmd_str, output=cmd['stdout'])

        return 0


@pytest.fixture
def mock_subprocess(monkeypatch):
    """Mock check_output, check_call from subprocess with data read from file or literal."""

    mock = MockSubprocess()

    monkeypatch.setattr(subprocess, 'check_output', mock.check_output)
    monkeypatch.setattr(subprocess, 'check_call', mock.check_call)
    return mock



"""Reference device for testing the individual report format
"""

from iotile.core.hw.virtual.virtualdevice import VirtualIOTileDevice
from iotile.core.hw.reports import IndividualReadingReport, IOTileReading


class RealtimeTestDevice(VirtualIOTileDevice):
    """Mock IOTileDevice that streams and traces data periodically

    This device can be configured to stream data on any streams at any interval.
    It can be used for testing realtime streaming functionality of any other portion
    of the IOTile stack.

    If no other arguments are passed, this device defaults to producing the value 100
    on stream 0x1001 every second.  If a streams dictionary is passed, that overrides
    this default setting.

    If no 'trace' argument is passed the device defaults to tracing the phrase
    'Hello trace world.  ' every second.  If a 'trace' array is passed, that overrides
    the default behavior.

    Args:
        args (dict): Any arguments that you want to pass to create this device.
            Supported args are:

                iotile_id (int)
                    The UUID used for this device.  If no UUID is specified, the default value of 1 is used.

                streams (dict):
                    A map of strings with hex numbers to tuples of (interval, value)
                    where interval is a float that expresses how often the stream should stream
                    in seconds and value is an integer that is sent as the value every interval
                    as a realtime reading (IndividualReadingReport).  The stream id is the key of the
                    streams dict which should be a string encoding of a hex number including
                    the prefix 0x so that it can be parsed with int(key, 0).

                trace (list):
                    A list of tuples which are (float, string) lists
                    that will trace the fixed string every fixed interval given
                    by the first float argument in seconds.
    """

    def __init__(self, args):
        iotile_id = args.get('iotile_id', 1)

        if isinstance(iotile_id, basestring) or isinstance(iotile_id, unicode):
            iotile_id = int(iotile_id, 16)

        super(RealtimeTestDevice, self).__init__(iotile_id, 'Simple')

        streams = {'0x1001': (1.0, 100)}

        if 'streams' in args:
            streams = args['streams']

        for key, value in streams.iteritems():
            stream = int(key, 0)
            interval, reading = value

            self.create_worker(self._create_stream, interval, stream, reading)

        traces = [[1.0, 'Hello trace world.  ']]

        if 'trace' in args:
            traces = args['trace']

        for interval, value in traces:
            self.create_worker(self._create_trace, interval, value)

    def _create_trace(self, value):
        """Send a realtime tracing value

        Args:
            value (string): The tracing value to send
        """

        if not self.trace_iface_open:
            return

        self.trace(bytearray(value.encode('ascii')))

    def _create_stream(self, stream, value):
        """Send a realtime streaming value

        Args:
            stream (int): The stream id to send
            value (int): The stream value to send
        """

        if not self.stream_iface_open:
            return

        reading = IOTileReading(0, stream, value)

        report = IndividualReadingReport.FromReadings(self.iotile_id, [reading])
        self.stream(report)
